--[[=============================================================================
    AVSwitch Protocol Initialization Functions

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end


PROTOCOL_DECLARATIONS = {}

function ON_DRIVER_EARLY_INIT.avswitch_init()

end

function ON_DRIVER_INIT.avswitch_init()
	--LogTrace("ON_DRIVER_INIT.ProtocolDeclarations()")
	for k,v in pairs(PROTOCOL_DECLARATIONS) do
		if (PROTOCOL_DECLARATIONS[k] ~= nil and type(PROTOCOL_DECLARATIONS[k]) == "function") then
			PROTOCOL_DECLARATIONS[k]()
		end
	end	
end

function ON_DRIVER_LATEINIT.avswitch_init()
end


function PROTOCOL_DECLARATIONS.CommandsTableInit_Serial()
	LogTrace("PROTOCOL_DECLARATIONS.CommandsTableInit_Serial()")

	CMDS = {
		--index:  Proxy Command Name
		--value:  Protocol Command Data
		
		--Power
		["ON"]             = "#power start=1",
		["OFF"]            = "#power shut=1",
	}
	
	CMDS[AVSWITCH_PROXY_BINDINGID] = {}
	
end

function PROTOCOL_DECLARATIONS.InputOutputTableInit()
	LogTrace("PROTOCOL_DECLARATIONS.InputOutputTableInit()")
	----------------------------------------- [*COMMAND/RESPONSE HELPER TABLES*] -----------------------------------------
	tInputCommandMap = {
		--index:  value of Output Connection id
		--value:  Protocol Command Data
		[1001] = "1",
		[1002] = "2",
		[1003] = "3",
		[1004] = "4",
		[1005] = "5",
		[1006] = "6",
		[1007] = "7",
		[1008] = "8",
	}
	tInputIdByNum = ReverseTable(tInputCommandMap)

	tOutputCommandMap = {
		--index:  value of Output Connection id
		--value:  Protocol Command Data
		[2001] = "1",
		[2002] = "2",
		[2003] = "3",
		[2004] = "4",
		[2005] = "5",
		[2006] = "6",
		[2007] = "7",
		[2008] = "8",
	}
	tOutputIdByNum = ReverseTable(tOutputCommandMap)
	
	tInputCommandMap = {
		--index:  Connection Name
		--value:  Protocol Command Data	
		--["ADAPTER PORT"] = "33", 
		["Input 1"] = "1",
		["Input 2"] = "2",
		["Input 3"] = "3",
		["Input 4"] = "4",
		["Input 5"] = "5",
		["Input 6"] = "6",
		["Input 7"] = "7",
		["Input 8"] = "8",
	}
	
	tInputResponseMap = ReverseTable(tInputCommandMap)	-- Reverses the tInputCommandMap table
			
	
		
	----------------------------------------- [*I/O HELPER TABLES*] -----------------------------------------
	tOutputConnMap = {
		--index:  value of Output Connection id
		--value:  Output Connection Name
		[2000] = "Video 1",
		[2001] = "Video 2",
		[2002] = "Video 3",
		[2003] = "Video 4",
		[2004] = "Video 5",
		[2005] = "Video 6",
		[2006] = "Video 7",
		[2007] = "Video 8",
		-- [4000] = "Audio 1",
		-- [4001] = "Audio 2",
		-- [4002] = "Audio 3",
		-- [4003] = "Audio 4",
		-- [4004] = "Audio 5",
		-- [4005] = "Audio 6",
		-- [4006] = "Audio 7",
		-- [4007] = "Audio 8",
		
	}

	tInputConnMapByID = {
		--index:  value of Input Connection id
		--[1000] = {Name = "INPUT HDMI 1",BindingID = AVSWITCH_PROXY_BINDINGID,},
		
		[1001] = {Name = "Input 1",BindingID = AVSWITCH_PROXY_BINDINGID,},
		[1002] = {Name = "Input 2",BindingID = AVSWITCH_PROXY_BINDINGID,},
		[1003] = {Name = "Input 3",BindingID = AVSWITCH_PROXY_BINDINGID,},
		[1004] = {Name = "Input 4",BindingID = AVSWITCH_PROXY_BINDINGID,},
		[1005] = {Name = "Input 5",BindingID = AVSWITCH_PROXY_BINDINGID,},
		[1006] = {Name = "Input 6",BindingID = AVSWITCH_PROXY_BINDINGID,},
		[1007] = {Name = "Input 7",BindingID = AVSWITCH_PROXY_BINDINGID,},
		[1008] = {Name = "Input 8",BindingID = AVSWITCH_PROXY_BINDINGID,},
	}
	
	tInputConnMapByName = {
		--index:  Input Connection Name
		--ID: value of Input Connection id
		--["INPUT HDMI 1"] = {ID = 0,BindingID = AVSWITCH_PROXY_BINDINGID,},
		["Input 1"] = {ID = 1001,BindingID = AVSWITCH_PROXY_BINDINGID,},
		["Input 2"] = {ID = 1002,BindingID = AVSWITCH_PROXY_BINDINGID,},
		["Input 3"] = {ID = 1003,BindingID = AVSWITCH_PROXY_BINDINGID,},
		["Input 4"] = {ID = 1004,BindingID = AVSWITCH_PROXY_BINDINGID,},
		["Input 5"] = {ID = 1005,BindingID = AVSWITCH_PROXY_BINDINGID,},
		["Input 6"] = {ID = 1006,BindingID = AVSWITCH_PROXY_BINDINGID,},
		["Input 7"] = {ID = 1007,BindingID = AVSWITCH_PROXY_BINDINGID,},
		["Input 8"] = {ID = 1008,BindingID = AVSWITCH_PROXY_BINDINGID,},
	}

end	

function ReverseTable(a)
	local b = {}
	for k,v in pairs(a) do b[v] = k end
	return b
end

