--[[=============================================================================
    Get, Handle and Dispatch message functions

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end
function ReceivedFromSerial(idBinding, strData)
    LogTrace("ReceivedFromSerial " .. idBinding .. " " .. strData)
    gReceiveBuffer = gReceiveBuffer .. strData
    ParsePacket()
end

function ParsePacket()
    local msg = GetMessage()
    while(msg ~= nil and msg ~= "") do
        HandleMessage(msg)
		msg = GetMessage()
	end
end
--[[=============================================================================
    GetMessage()
  
    Description:
    Used to retrieve a message from the communication buffer. Each driver is
    responsible for parsing that communication from the buffer.
  
    Parameters:
    None
  
    Returns:
    A single message from the communication buffer
===============================================================================]]
function GetMessage()
	local message, pos
	
	--TODO: Implement a string using Lua captures and patterns which 
	--		will be used by string.match to parse out a single message
	--      from the receive buffer(gReceiveBuffer).
	--		The example shown here will return all characters from the beginning of 
	--		the gReceiveBuffer up until but not including the first carriage return.
	local pattern = "^(.-)\n()"
	
	if (gReceiveBuffer:len() > 0) then
		message, pos = string.match(gReceiveBuffer, pattern)
		if (message == nil) then
			--LOG:Info("Do not have a complete message")
			return ""
		end
		gReceiveBuffer = gReceiveBuffer:sub(pos)		
	end

	return message
	
end

--[[=============================================================================
    HandleMessage(message)]

    Description
    This is where we parse the messages returned from the GetMessage()
    function into a command and data. The call to 'DispatchMessage' will use the
    'name' variable as a key to determine which handler routine, function, should
    be called in the DEV_MSG table. The 'value' variable will then be passed as
    a string parameter to that routine.

    Parameters
    message(string) - Message string containing the function and value to be sent to
                      DispatchMessage

    Returns
    Nothing
===============================================================================]]

function HandleMessage(message)
	if (type(message) == "table") then
		LogTrace("HandleMessage. Message is ==>")
		LogTrace(message)
		LogTrace("<==")
	else
		LogTrace("HandleMessage. Message is ==>%s<==", message)
	end
	local routingPattern =  "<out(%d) video matrix>  Data:(%d)%."
	if(message:find(routingPattern)) then
		LogDebug("Found routing feedback pattern")
		local o, i = string.match(message, routingPattern)
		proxyFeedbackRoutingState(1000 + i, 2000 + o)
	elseif message ==  "Self-check is completed." then
		LogDebug("Found power on pattern")
		proxyFeedbackPower(true)
	elseif message == "System standby." then
		LogDebug("Found power off pattern")
		proxyFeedbackPower(false)
	else
		LogDebug("No pattern found")
	end
end

function proxyFeedbackRoutingState(input_id,output_id)
	gAVSwitchProxy:dev_InputOutputChanged(input_id, output_id)
end

gState = false
function proxyFeedbackPower(state)
    -- TODO: derive and set  "output" from value or create separate DEV_MSG functions for each Output Connection
    local output = 0 	--mod 1000 value of Output Connection ID
     gState = state   
    if (state == true) then
        gAVSwitchProxy:dev_PowerOn(output)
    elseif (state== false) then
		gAVSwitchProxy:dev_PowerOff(output)
	end
end